from typing import Optional


class RuptureProduct(object):
    def __init__(
        self,
        product_code,           # type: str
        product_name,           # type: str
        products_composition,   # type: str
        product_quantity=0      # type: Optional[int]
    ):
        # type: (...) -> None

        self.product_code = product_code
        self.product_name = product_name
        self.products_composition = products_composition
        self.product_quantity = product_quantity
