from application.service import LoaderUpdaterService


class LoaderUpdaterInteractor(object):
    def __init__(self, loader_updater_service):
        # type: (LoaderUpdaterService) -> None # noqa

        self.loader_updater_service = loader_updater_service

    def update_loaders(self):
        # type: () -> None

        return self.loader_updater_service.update_loaders(True)

