from typing import Optional


class RuptureItem(object):
    def __init__(
        self,
        product_code,       # type: str
        product_name,       # type: str
        product_part_code,  # type: str
        product_quantity=0  # type: Optional[int]
    ):
        # type: (...) -> None

        self.product_code = product_code
        self.product_name = product_name
        self.product_part_code = product_part_code
        self.product_quantity = product_quantity
