from typing import Union


class AposA8PrintLine(object):
    def __init__(
        self,
        align,                  # type: int
        text,                   # type: Union[str, unicode]
        command_type,           # type: int
        font_size="small",      # type: str
        bold=False,             # type: bool
    ):
        # type: (...) -> None

        self.align = align
        self.text = text
        self.command_type = command_type
        self.font_size = font_size.lower()
        self.bold = bold
