import json


class ChangeProdStateRequest(object):
    def __init__(
        self,           # type: str
        order_id,       # type: str
        view,           # type: str
        state,          # type: str
        pickup_info,    # type: json
    ):
        # type: (...) -> None

        self.order_id = order_id
        self.view = view
        self.state = state
        self.pickup_info = json.loads(pickup_info) if pickup_info else pickup_info
