from typing import Optional


class Store(object):
    def __init__(
            self,
            crt=None,                   # type: Optional[int]
            name=None,                  # type: Optional[unicode]
            cnpj=None,                  # type: Optional[unicode]
            state_registration=None,    # type: Optional[unicode]
            postal_code=None,           # type: Optional[unicode]
            address_street=None,        # type: Optional[unicode]
            address_number=None,        # type: Optional[unicode]
            address_complement=None,    # type: Optional[unicode]
            neighborhood=None,          # type: Optional[unicode]
            city=None,                  # type: Optional[unicode]
            state=None,                 # type: Optional[unicode]
            city_code=None,             # type: Optional[int]
            state_code=None,            # type: Optional[int]
    ):

        # type: (...) -> None

        self.name = name
        self.crt = crt
        self.cnpj = cnpj
        self.state_registration = state_registration
        self.postal_code = postal_code
        self.address_street = address_street
        self.address_number = address_number
        self.address_complement = address_complement
        self.neighborhood = neighborhood
        self.city = city
        self.state = state
        self.city_code = city_code
        self.state_code = state_code
